--[[ Local                                                            ]]
local _G = getfenv(0);
local LibStub = _G["LibStub"];
local AddOn = LibStub("AceAddon-3.0"):GetAddon("Enhancer");
local lego = LibStub:GetLibrary("LegoBlock-Beta1");

local lcase, gsub, trim = _G.string.lower, _G.string.gsub, _G.strtrim;
local assert = _G.assert;

local legoBackdrop = { bgFile = [[Interface/Icons/Spell_Totem_WardOfDraining]], edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = false, tileSize = 0, edgeSize = 16, insets = { left = 3, right = 3, top = 3, bottom = 3 } };

function AddOn:CreateButton(gname, options)
	assert(gname, "Can't create a nameless button!");
	assert(options, "Need options table to create button!");
	
	local retVal = {};
	retVal.Texts = {};
	
	-- Save important default values
	retVal.dValues = {};
	retVal.dValues.icon = legoBackdrop.bgFile;
	options.width = self.db.profile.btnSize;
	options.height = self.db.profile.btnSize;
	
	-- Create a Secure Button we can mash if we want
	retVal.Button = lego:New("EnhancerButton_"..gname, "", nil, options);
	retVal.Button:SetBackdrop(legoBackdrop); -- Make it pretty
	retVal.Button:SetBackdropBorderColor(1, 1, 1, 0); -- Hide the border
	retVal.Button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	local globalname = retVal.Button:GetName();
	local bWidth = retVal.Button:GetWidth();
	
	-- Create a Cooldown on top of it
	retVal.Cooldown = CreateFrame("Cooldown", "EnhancerCooldown_"..gname, retVal.Button, "CooldownFrameTemplate");
	retVal.Cooldown:ClearAllPoints();
	retVal.Cooldown:SetWidth(bWidth * 0.75);
	retVal.Cooldown:SetHeight(bWidth * 0.75);
	retVal.Cooldown:SetPoint("CENTER", retVal.Button, "CENTER", 0, 0);
	--retVal.Cooldown:SetAllPoints(globalname);
	
	-- Create a text holder in the center
	retVal.Texts.Center = retVal.Button:CreateFontString("EnhancerTextCenter"..gname, "OVERLAY");
	retVal.Texts.Center:SetFontObject(GameFontHighlightSmall)
	retVal.Texts.Center:SetFont(self.db.profile.centerFontIDTTF, self.db.profile.centerFontSize, self.db.profile.centerFontFlags);
	retVal.Texts.Center:ClearAllPoints();
	retVal.Texts.Center:SetTextColor(1, 1, 1, 1);
	retVal.Texts.Center:SetWidth(bWidth);
	retVal.Texts.Center:SetHeight(bWidth);
	retVal.Texts.Center:SetPoint("CENTER", globalname, "CENTER");
	retVal.Texts.Center:SetJustifyH("CENTER");
	retVal.Texts.Center:SetJustifyV("MIDDLE");
	retVal.Texts.Center:SetShadowColor(0, 0, 0, 1);
	retVal.Texts.Center:SetShadowOffset(0.8, -0.8);
	retVal.Texts.Center:SetText("");
	
	-- Create a text holder at the top
	retVal.Texts.Top = retVal.Button:CreateFontString("EnhancerTextTop"..gname, "OVERLAY");
	retVal.Texts.Top:SetFontObject(GameFontHighlightSmall)
	retVal.Texts.Top:SetFont(self.db.profile.topFontIDTTF, self.db.profile.topFontSize, self.db.profile.topFontFlags);
	retVal.Texts.Top:ClearAllPoints();
	retVal.Texts.Top:SetTextColor(1, 1, 1, 1);
	retVal.Texts.Top:SetWidth(bWidth * 1.5);
	retVal.Texts.Top:SetHeight(self.db.profile.topFontSize + self.db.profile.topFontSize + 10);
	retVal.Texts.Top:SetPoint("BOTTOM", globalname, "TOP");
	retVal.Texts.Top:SetJustifyH("CENTER");
	retVal.Texts.Top:SetJustifyV("BOTTOM");
	retVal.Texts.Top:SetShadowColor(0, 0, 0, 1);
	retVal.Texts.Top:SetShadowOffset(0.8, -0.8);
	retVal.Texts.Top:SetText("");
	
	-- Create a text holder at the bottom
	retVal.Texts.Bottom = retVal.Button:CreateFontString("EnhancerTextBottom"..gname, "OVERLAY");
	retVal.Texts.Bottom:SetFontObject(GameFontHighlightSmall)
	retVal.Texts.Bottom:SetFont(self.db.profile.bottomFontIDTTF, self.db.profile.bottomFontSize, self.db.profile.bottomFontFlags);
	retVal.Texts.Bottom:ClearAllPoints();
	retVal.Texts.Bottom:SetTextColor(1, 1, 1, 1);
	retVal.Texts.Bottom:SetWidth(bWidth * 1.5);
	retVal.Texts.Bottom:SetHeight(self.db.profile.bottomFontSize + self.db.profile.bottomFontSize + 10);
	retVal.Texts.Bottom:SetPoint("TOP", globalname, "BOTTOM");
	retVal.Texts.Bottom:SetJustifyH("CENTER");
	retVal.Texts.Bottom:SetJustifyV("TOP");
	retVal.Texts.Bottom:SetShadowColor(0, 0, 0, 1);
	retVal.Texts.Bottom:SetShadowOffset(0.8, -0.8);
	retVal.Texts.Bottom:SetText("");
	
	-- Create some functions
	retVal.SetAlpha = function(self, alpha)
		self.Button:SetAlpha(alpha);
		self.Cooldown:SetAlpha(alpha);
		--self.Texts.Center:SetAlpha(alpha);
		--self.Texts.Top:SetAlpha(alpha);
		--self.Texts.Bottom:SetAlpha(alpha);
	end
	
	retVal.Resize = function(self, db)
		self.Button:SetWidth(db.btnSize);
		self.Button:SetHeight(db.btnSize);
		self.Cooldown:SetWidth(db.btnSize * 0.75);
		self.Cooldown:SetHeight(db.btnSize * 0.75);
		self.Texts.Center:SetWidth(db.btnSize);
		self.Texts.Center:SetHeight(db.btnSize);
		self.Texts.Top:SetWidth(db.btnSize * 1.5);
		self.Texts.Bottom:SetWidth(db.btnSize * 1.5);
		
		self.Texts.Center:SetFont(db.centerFontIDTTF, db.centerFontSize, db.centerFontFlags);
		self.Texts.Top:SetFont(db.topFontIDTTF, db.topFontSize, db.topFontFlags);
		self.Texts.Top:SetHeight(db.topFontSize + db.topFontSize + 10);
		self.Texts.Bottom:SetHeight(db.bottomFontSize + db.bottomFontSize + 10);
		self.Texts.Bottom:SetFont(db.bottomFontIDTTF, db.bottomFontSize, db.bottomFontFlags);
	end
	
	retVal.Show = function(self)
		self.Button:Show();
		self.Cooldown:Show();
		self.Texts.Center:Show();
		self.Texts.Top:Show();
		self.Texts.Bottom:Show();
	end
	
	retVal.Hide = function(self)
		self.Button:Hide();
		self.Cooldown:Hide();
		self.Texts.Center:Hide();
		self.Texts.Top:Hide();
		self.Texts.Bottom:Hide();
	end
	
	retVal.SetScript = function(self, event, script)
		self.Button:SetScript(event, script);
	end
	
	retVal.SetIcon = function(self, icon)
		local backdrop = self.Button:GetBackdrop();
		backdrop.bgFile = icon;
		self.Button:SetBackdrop(backdrop);
		self.Button:SetBackdropBorderColor(1, 1, 1, 0);
	end
	
	retVal.SetMacro = function(self, macro, button)
		button = button or 1;
		self.Button:SetAttribute("type" .. button, "macro");
		self.Button:SetAttribute("macrotext" .. button, macro);
	end
	
	retVal.SetText = function(self, text, location)
		location = location or "Center";
		self.Texts[location]:SetText(trim(text));
	end
	
	retVal.SetCooldown = function(self, start, duration) 
		self.Cooldown:SetCooldown(start, duration);
	end
	
	retVal.storage = {};
	retVal.Get = function(self, key)
		return self.storage and self.storage[lcase(key)];
	end
	retVal.Set = function(self, key, value)
		self.storage = self.storage or {};
		self.storage[lcase(key)] = value;
	end
	retVal.Reset = function(self)
		local backdrop = self.Button:GetBackdrop();
		backdrop.bgFile = legoBackdrop.bgFile;
		self.Button:SetBackdrop(backdrop);
		self.Button:SetBackdropBorderColor(1, 1, 1, 0);
		self.storage = nil;
		
		self.Texts.Bottom:SetText("");
		self.Texts.Center:SetText("");
	end
	
	retVal.Lock = function(self, ...)
		if (select("#", ...) > 0) then
			self.Button.optionsTbl.locked = select("1", ...);
		else
			self.Button.optionsTbl.locked = not self.Button.optionsTbl.locked;
		end
	end
	
	retVal.SavePos = function(self)
		local func = self.Button:GetScript("OnDragStop");
		self.Button.isMoving = true;
		return xpcall(func, self.Button);
	end
	
	retVal.IsLocked = function(self)
		return self.Button.optionsTbl.locked;
	end
	
	retVal.Info = function(self, key)
		return AddOn.TotemInfo[self:Get("name")] and AddOn.TotemInfo[self:Get("name")][key];
	end
	retVal.RankInfo = function(self, key)
		return AddOn.TotemInfo[self:Get("name")] and AddOn.TotemInfo[self:Get("name")][self:Get("rank")] and AddOn.TotemInfo[self:Get("name")][self:Get("rank")][key];
	end
	
	retVal.format = function(self, text)
		return (gsub(text,"%%(.-)%%", function(a) if (lcase(a) == "element") then return AddOn.slotElement[self.Slot] else return tostring(self:Get(a)) end end ))
	end
	
	retVal.Button:SetScript("OnEnter", function() GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT"); end);
	retVal.Button:SetScript("OnLeave", function() GameTooltip:Hide(); end);
	
	return retVal;
end

--[[
item:SetAttribute("type1", "item")
item:SetAttribute("item1", "item:"..uitem)

f:SetAttribute("type1", "spell")
f:SetAttribute("spell1", "Lightning Bolt")

f:SetAttribute("type1", nil)
f:SetAttribute("spell1", nil)
]]